/*
 * Raima Database Manager
 *
 * Copyright (c) 2019 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/*! \file
 * \brief Source code for the RDM Perf05 example
 */
#include <stdio.h>
#include "perftimer.h"
#include "print_error.h"
#include "rdm.h"

/* Generated \c struct and \c typedef definitions to be used with the RDM APIs */
#include "perf05_db_structs.h"

/* Generated catalog definition to be used with the RDM rdm_dbSetCatalog() API */
#include "perf05_db_cat.h"

/*! Number of records to insert */
#define NUM_RECORDS 50000

/*!
 * \mainpage Perf05 Popcorn Example
 *
 * The Perf05 example performs a number of atomic database operations
 * on a very simple database schema using the RDM Core C APIs.
 *
 * Each time you run this example
 *   * The database is initialized (removing all existing data)
 *   * 50,000 new records are created using different transaction block sizes
 *       * 50,000
 * 10,000
 *  5,000
 *  1,000
 *    500
 *    100
 *   * Database is cleaned up
 *
 * \par Table of Contents
 *
 * - \subpage hDB
 * - \subpage hRDMfunc
 * - \subpage hPGMfunc
 *
 * For additional information, please refer to the product documentation at http://docs.raima.com/.
 *
 * \page hDB Database Schema
 *
 * \par Database Schema Definition
 *
 * The DDL (Database Definition Language) specification for the database used
 * in this example is located in the file \c perf05_db.sdl. The example utilizes
 * the SQL DDL syntax and specifies database schema with a single table (SIMPLE).
 * The table has a single column (INT_COL). There are no index definitions for
 * this database.
 *
 * \include perf05_db.sdl
 *
 * The schema was compiled using the RDM rdm-compile utility with the -s and -a options to
 * generate an embedded database dictionary.
 *
 * \code rdm-compile -s -a perf05_db.sdl \endcode
 *
 * The schema catalog information (\c perf05_db_cat.c) is embedded inside the application.
 *
 * \page hRDMfunc RDM Functions Used
 * The following RDM functions are illustrated:
 *   * rdm_rdmAllocTFS/rdm_tfsFree
 *   * rdm_tfsAllocDatabase/rdm_dbFree
 *   * rdm_dbSetCatalog
 *   * rdm_dbOpen/rdm_dbClose
 *   * rdm_dbInsertRow
 *   * rdm_retcodeGetName
 *   * rdm_retcodeGetDescription
 *
 * They are fully documented in http://docs.raima.com/.
 *
 * \page hPGMfunc Program Functions
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * \li openEmptyDatabase() - \copybrief openEmptyDatabase
 * \li cleanup() - \copybrief cleanup
 * \li add_records() - \copybrief add_records
 * \li delete_records() - \copybrief delete_records
 * \li main() - \copybrief main
 */
/*! \brief Initialize the RDM runtime library for use in the perf05 example
 *
 * This function initializes the RDM Transactional File Server (TFS) to use
 * the embedded implementation.  It also creates an RDM database handle
 * and opens the "perf05" database in exclusive mode.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE openEmptyDatabase (RDM_TFS *pTFS, /**< [out] Pointer to the TFS handle */
                               RDM_DB *pDB)   /**< [out] Pointer to the RDM database handle */
{
    RDM_RETCODE rc;

    rc = rdm_rdmAllocTFS ("tfstype=EMBED", pTFS);
    print_error (rc);

    if (rc == sOKAY)
    {
        /* If we have a TFS handle ... */
        rc = rdm_tfsAllocDatabase (*pTFS, pDB);
        print_error (rc);

        if (rc == sOKAY)
        {
            /* If we have a DB handle ... */
            rc = rdm_dbSetCatalog (*pDB, perf05_db_cat);
            print_error (rc);

            if (rc == sOKAY)
            {
                rc = rdm_dbSetOptions (*pDB, "storage=inmemory_volatile");
                print_error (rc);
            }
            if (rc == sOKAY)
            {
                rc = rdm_dbOpen (*pDB, "perf05", RDM_OPEN_EXCLUSIVE);
                print_error (rc);
            }
            if (rc == sOKAY)
            {
                rc = rdm_dbDeleteAllRowsFromDatabase (*pDB);
                print_error (rc);
            }

            /* If any of the above DB operations failed, free the DB handle */
            if (rc != sOKAY)
            {
                rdm_dbFree (*pDB);
            }
        }

        /* If any of the above DB operations failed, free the TFS handle */
        if (rc != sOKAY)
        {
            rdm_tfsFree (*pTFS);
        }
    }
    return rc;
}

/*! \brief Cleanup the RDM runtime library
 *
 * This functions closes all open databases and cleans up the RDM runtime task
 * used in the perf01 example.  It also terminates the STANDALONE TFS used
 * in the example.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
void cleanup (RDM_TFS hTFS, /**< [out] Pointer to the TFS handle to be terminated */
              RDM_DB hDB)   /**< [out] Pointer to the DB handle to be terminated */
{
    /* close the database */
    rdm_dbClose (hDB);

    /* free the database handle */
    rdm_dbFree (hDB);

    /* free the TFS handle */
    rdm_tfsFree (hTFS);
}

/*! \brief Remove simple records into the perf05 database
 *
 * This function empties the simple table database.  The
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE delete_records (RDM_DB hDB) /**< [in] RDM db handle with the database open */
{
    RDM_RETCODE rc;

    /* empty the table */
    rc = rdm_dbDeleteAllRowsFromTable (hDB, TABLE_SIMPLE);
    print_error (rc);
    return rc;
}

/*! \brief Insert 50,000 simple records into the perf05 database
 *
 * This function adds 50,000 simple records into the inmemory perf database.  The
 * simple record contains only a single integer field with no index.  The function will
 * calculate and display the total time elapsed while inserting the records
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE add_records (unsigned int trans_size, /**< [in] Transaction block size */
                         RDM_DB hDB)              /**< [in] RDM db handle with the database open */
{
    RDM_RETCODE rc = sOKAY;
    SIMPLE simple_rec = {0};
    perfTimer_t timer;
    unsigned int count = 0;
    unsigned int ii;

    printf ("Add %d records with %-5d per transaction:\t", NUM_RECORDS, trans_size);

    timeMeasureBegin (&timer);

    /* Loop to add 50,000 new simple record */
    for (ii = 0, count = 0; ii < NUM_RECORDS && rc == sOKAY; ii++)
    {
        if (count == 0)
        {
            /* Start an update transaction and lock the table */
            rc = rdm_dbStartUpdate (hDB, RDM_LOCK_ALL, 0, NULL, 0, NULL);
            print_error (rc);
        }

        if (rc == sOKAY)
        {
            /* Set values for the new record */
            simple_rec.INT_COL = 1;

            /* Add the record to the database */
            rc = rdm_dbInsertRow (hDB, TABLE_SIMPLE, &simple_rec, sizeof (simple_rec), NULL);
            print_error (rc);
        }

        if (rc == sOKAY)
        {
            /* Is it time to commit the transaction? */
            if (++count == trans_size)
            {
                rc = rdm_dbEnd (hDB);
                print_error (rc);
                count = 0;
            }
        }
    }
    timeMeasureEnd (&timer);

    printf ("%u milliseconds\n", timeMeasureDiff (&timer));

    return rc;
}

/*! \brief Main function for perf05 example
 *
 * The function initializes the RDM environment and runs the insert operations.
 * over a number of transaction block sizes
 *
 * @return Returns the RDM_RETCODE on exit.
 */
int main ()
{
    RDM_DB hDB;
    RDM_TFS hTFS;
    RDM_RETCODE rc;
    int ii;
    unsigned int trans_size[] = {50000, 10000, 5000, 1000, 500, 100};

    /* Display the example title */
    printf ("%s\n", RDM_EXAMPLE_DESC ("Perf05"));

    /* Initialize the TFS, task, and open/initialize the database */
    rc = openEmptyDatabase (&hTFS, &hDB);
    if (rc == sOKAY)
    {
        for (ii = 0; rc == sOKAY && ii < (sizeof (trans_size) / sizeof (trans_size[0])); ii++)
        {
            /* delete all records before testing insert */
            rc = delete_records (hDB);

            if (rc == sOKAY)
            {
                /* Add records to the database */
                rc = add_records (trans_size[ii], hDB);
            }
        }
        cleanup (hTFS, hDB);
    }
    return (int) rc;
}
