/*
 * Raima Database Manager
 *
 * Copyright (C) 2019 Raims Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/** \file rdmcmdlinetypes.h
 *  \brief Header for command line parsing types
 */

#include "psptypes.h"

#if !defined (__RDMCMDLINETYPES_H_INCLUDED_)
#define __RDMCMDLINETYPES_H_INCLUDED_

/** \defgroup rdm_cmd Command line parsing API
 *
 *  The RDM command line API.  The functions here are located in \lib{base}
 *
 * \ingroup rdm
 * \addtogroup rdm_cmd
 * @{
 */

/** \brief Generic usage function option record */
typedef struct
{
    const char
        *optShort;       /**< Short option is a single character used to match
                          * against. A short option is a command line option that
                          * starts with '-'       followed by a sequence of characters
                          * where       one of these characters are the one specified
                          * here.       rdm_cmdlineNextShortOption       (and
                          * rdm_cmdlineNextLongOption       when optLong is NULL) will
                          * return this string for a match.       <p>       The last
                          * element       in       the array of command line       parsing
                          * options should       have       optShort set to NULL. */
    const char *optLong; /**< Long option is a string of at least a few
                          * characters used to match against. A long option is a
                          * command line option that starts with '--' followed
                          * by the exact sequence of characters specified here.
                          *   <p>
                          *   rdm_cmdlineNextLongOption will return this string
                          * for a match except when it is NULL. */
    const char
        *arg;         /**< Option argument or argument.
                       *   <p>
                       *   An option that takes arguments have an entry here with any of
                       * these formats: <ul> <li>s =... for a string option
                       * argument,</li> <li>i=... for an integer option argument,
                       * and</li> <li>c=. for a single character option argument.</li>
                       *   </ul>
                       *   <p>
                       *   A standalone argument have an entry here with any of these
                       * formats: <ul> <li>#=... for a single command line
                       * argument,</li> <li>*=... for zero or more command line
                       * arguments, and</li> <li>?=... for an optional argument.</li>
                       *   </ul>
                       *   It only makes sense to have one '*=...' and it should always
                       * be the second last element in the array of command line parsing
                       * options.  The string after the equal sign (shown above as
                       * '...') is a documentation string for the argument. */
    const char *desc; /**< Option or argument description.  This string is only
                       * for documentation purposes and will be printed with the
                       * '-h' or '--help' option. */
} RDM_CMDLINE_OPT;

#if defined(RDM_DOXYGEN)
/** \brief The buffer used by the command line parser to hold state information
 *
 *  Declare this struct without any initialization and pass the
 *  address of it to \ref rdm_cmdlineInit and other command line
 *  functions.
 *
 *  \see rdm_cmdlineInit
 *  \see rdm_cmdlineNextLongOption
 *  \see rdm_cmdlineNextShortOption
 */
typedef struct
{
} RDM_CMDLINE;
#else
typedef struct _RDM_CMDLINE_DEFAULTS RDM_CMD_DEFAULTS;

typedef struct
{
    uint32_t ii;
    uint32_t ii_next;
    uint32_t kk;
    uint32_t argument_ind_next;

    const RDM_CMDLINE_OPT *opts;
    int32_t argc;
    const char *const *argv;

    const char *banner;
    const char *description;

    const RDM_CMD_DEFAULTS *defaults;
} RDM_CMDLINE;
#endif

/** @} */
#endif /* __RDMCMDLINETYPES_H_INCLUDED_ */
