/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2010 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmdatetimetypes.h
 *  \brief Header for the public Date/Time types
 */

#if !defined(RDMDATETIMETYPES_H_INCLUDED_)
#define RDMDATETIMETYPES_H_INCLUDED_

#include "psptypes.h"

/** \brief The Maximum date */
#define RDM_DATE_MAX RDM_UINT32_MAX
/** \brief The Maximum time */
#define RDM_TIME_MAX ((((((23 * 60) + 59) * 60) + 59) * 10000) + 9999)

/** \brief The Maximum time presision 1/x'th of a second */
#define MAX_TIME_PREC 10000

/** \brief The default date format */
#define RDM_DEF_DATE_FORMAT RDM_YYYYMMDD
/** \brief The default date separator */
#define RDM_DEF_DATE_SEP '-'
/** \brief The default time format */
#define RDM_DEF_TIME_FORMAT RDM_HHMMSSFFFF

typedef char RDM_DATE_SEPARATOR;

/** \brief The RDM Date data structure
 */
typedef struct
{
    int32_t year;   /**< The year in the range 1 - 9999 */
    uint16_t month; /**< The month in the range 1 - 12 */
    uint16_t day;   /**< The date in the range 1 - 31 */
} RDM_DATE_T;

/** \brief The RDM Time data structure
 */
typedef struct
{
    uint16_t hour;   /**< The hour in the range 0 - 23 */
    uint16_t minute; /**< The minute in the range 0 - 59 */
    uint16_t second; /**< The second in the range 0 - 59 */
    uint16_t fraction; /**< The 1/1000th fraction of a second in the range 0 - 9999 */
} RDM_TIME_T;

/** \brief The RDM Timestamp data structure
 */
typedef struct
{
    int32_t year;    /**< The year in the range 1 - 9999 */
    uint16_t month;  /**< The month in the range 1 - 12 */
    uint16_t day;    /**< The date in the range 1 - 31 */
    uint16_t hour;   /**< The hour in the range 0 - 23 */
    uint16_t minute; /**< The minute in the range 0 - 59 */
    uint16_t second; /**< The second in the range 0 - 59 */
    uint16_t fraction; /**< The 1/1000th fraction of a second in the range 0 - 9999 */
} RDM_TIMESTAMP_T;

#if 0
/** \brief The RDM Date w/ timezone data structure
 */
typedef struct
{
    int32_t  year;      /**< The year in the range 1 - 9999 */
    uint16_t month;     /**< The month in the range 1 - 12 */
    uint16_t day;       /**< The date in the range 1 - 31 */
    int16_t  tz;        /**< Time Zone in minutes offset from UTC [-14 * 60 - +14 * 60] */
} RDM_DATETZ_T;
#endif

/** \brief The RDM Time w/ timezone data structure
 */
typedef struct
{
    uint16_t hour;   /**< The hour in the range 0 - 23 */
    uint16_t minute; /**< The minute in the range 0 - 59 */
    uint16_t second; /**< The second in the range 0 - 59 */
    uint16_t fraction; /**< The 1/1000th fraction of a second in the range 0 - 9999 */
    int16_t tz; /**< Time Zone in minutes offset from UTC [-14 * 60 - +14 * 60] */
} RDM_TIMETZ_T;

/** \brief The RDM Timestamp data structure
 */
typedef struct
{
    int32_t year;    /**< The year in the range 1 - 9999 */
    uint16_t month;  /**< The month in the range 1 - 12 */
    uint16_t day;    /**< The date in the range 1 - 31 */
    uint16_t hour;   /**< The hour in the range 0 - 23 */
    uint16_t minute; /**< The minute in the range 0 - 59 */
    uint16_t second; /**< The second in the range 0 - 59 */
    uint16_t fraction; /**< The 1/1000th fraction of a second in the range 0 - 9999 */
    int16_t tz; /**< Time Zone in minutes offset from UTC [-14 * 60 - +14 * 60] */
} RDM_TIMESTAMPTZ_T;

/** \brief The RDM packed Date data structure
 */
typedef uint32_t RDM_PACKED_DATE_T;

/** \brief The RDM packed Time data structure
 */
typedef uint32_t RDM_PACKED_TIME_T;

/** \brief The RDM packed Timestamp data structure
 */
typedef struct
{
    RDM_PACKED_DATE_T date; /**< The data part */
    RDM_PACKED_TIME_T time; /**< The time part */
} RDM_PACKED_TIMESTAMP_T;

/** \brief The RDM packed Time with time zone data structure
 */
typedef struct
{
    RDM_PACKED_TIME_T time; /**< The time part */
    int16_t tz;             /**< The timezone part */
} RDM_PACKED_TIMETZ_T;

/** \brief The RDM packed Timestamp with time zone data structure
 */
typedef struct
{
    RDM_PACKED_DATE_T date; /**< The data part */
    RDM_PACKED_TIME_T time; /**< The time part */
    int16_t tz;             /**< The timezone part */
} RDM_PACKED_TIMESTAMPTZ_T;

/* clang-format off */
/* The comments need to be on one line as we parse this file to generate RsqlEnums.cs */
/** \brief RDM Date format codes */
typedef enum
{
    /* GEN_RDM_DATE_FORMAT_BEGIN */
    RDM_MMDDYYYY = 1, /**< The date format code for month, followed by a date separator, followed by date, followed by a date separator, followed by year */
    RDM_YYYYMMDD, /**< The date format code for year, followed by a date separator, followed by month, followed by a date separator, followed by date */
    RDM_DDMMYYYY /**< The date format code for date, followed by a date separator, followed by month, followed by a date separator, followed by year */
    /* GEN_RDM_DATE_FORMAT_END */
} RDM_DATE_FORMAT;
/* clang-format on */

/* clang-format off */
/* The comments need to be on one line as we parse this file to generate RsqlEnums.cs */
/** \brief RDM time format codes */
typedef enum
{
    /* GEN_RDM_TIME_FORMAT_BEGIN */
    RDM_HH = 1, /**< The time format code for hours */
    RDM_HHMM, /**< The time format code for hours, followed by ':', followed by the time separator, TBD followed by minutes */
    RDM_HHMMSS,   /**< The time format code for hours, followed by ':', followed by minutes, followed by ':', followed by seconds */
    RDM_HHMMSSF,  /**< The time format code for hours, followed by ':', followed by minutes, followed by ':', followed by seconds, followed by '.', followed by 1/10th of seconds */
    RDM_HHMMSSFF, /**< The time format code for hours, followed by ':', followed by minutes, followed by ':', followed by seconds, followed by '.', followed by 1/100th of seconds */
    RDM_HHMMSSFFF, /**< The time format code for hours, followed by ':', followed by minutes, followed by ':', followed by seconds, followed by '.', followed by 1/1000th of seconds */
    RDM_HHMMSSFFFF /**< The time format code for hours, followed by ':', followed by minutes, followed by ':', followed by seconds, followed by '.', followed by 1/10000th of seconds */
    /* GEN_RDM_TIME_FORMAT_END */
} RDM_TIME_FORMAT;
/* clang-format on */

#endif /* RDMDATETIMETYPES_H_INCLUDED_ */
