/*
 * Raima Database Manager
 *
 * Copyright (C) 2016 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/** \file rdmuuidapi.h
 *  \brief Header for Universally Unique Identifier (UUID) API
 */

#include "rdmuuidtypes.h"
#include "rdmretcodetypes.h"

#if defined(RDM_UNICODE)
#define rdm_uuidFromString RDM_T_FUNC (rdm_uuidFromString)
#define rdm_uuidToString RDM_T_FUNC (rdm_uuidToString)
#endif

/** \defgroup rdm_uuid Universally Unique Identifier (UUID) API
 * \ingroup datatypes
 *
 * The RDM uuid datatype API.  The functions here are located in \lib{base}
 *
 * The UUID/GUID types represent 128-bit unique identifiers. The
 * implementation of UUID/GUID is represented by the \html{RFC 4122
 * standard, http://tools.ietf.org/html/rfc4122}
 *
 * \addtogroup rdm_uuid
 * @{
 */

#if defined(__cplusplus)
extern "C" {
#endif

/** Create a Universally Unique Identifier (UUID)
 *
 *  This function is able to create a universally unique identifier
 *  with a very high probability.
 *
 *  \see rdm_uuidToString
 *  \see rdm_uuidFromString
 */
RDM_EXPORT void EXTERNAL_FCN rdm_uuidCreate (
    RDM_UUID_T *uuid /**< [OUT] The Universally Unique Identifier to be creasted */
);

/** Convert a string to a Universally Unique Identifier (UUID)
 *
 * Convert a string to a Universally Unique Identifier (UUID).
 *
 *  \retval{sOKAY}
 *  \retval{ePSP_INVUUID}
 *
 *  \see rdm_uuidToString
 *  \see rdm_uuidCreate
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_uuidFromString (
    const RDM_TCHAR_T
        *str, /**< [IN] The UUID in string form that is to be converted */
    RDM_UUID_T *uuid /**< [OUT] The converted UUID */
);

/** Get the string length of a Universally Unique Identifier (UUID)
 *
 * Get the string length of a Universally Unique Identifier if it were
 * to be converted to a string.
 *
 * \return The string length, if converted to a string
 */
RDM_EXPORT size_t EXTERNAL_FCN rdm_uuidStrLen (
    const RDM_UUID_T *
        uuid, /**< [IN] The UUID we want the string length off if converted to a string */
    RDM_BOOL_T braces /**< [IN] Whether the converted string should include braces */
);

/** Convert a Universally Unique Identifier (UUID) to a string
 *
 *  Convert a Universally Unique Identifier (UUID) to a string.  The
 *  string buffer provided should be big enough to hold at least 36
 *  characters, plus the optional braces, and a null terminator.
 *
 *  \retval{sOKAY}
 *  \retval{sTRUNCATE}
 *
 *  \see rdm_uuidStrLen
 *  \see rdm_uuidFromString
 *  \see rdm_uuidCreate
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_uuidToString (
    const RDM_UUID_T *uuid, /**< [IN] The UUID that is to be converted */
    RDM_TCHAR_T *str, /**< [OUT] The string buffer for the converted UUID */
    size_t len, /**< [IN] The lengt of the provided string buffer measured in number of characters */
    RDM_BOOL_T braces /**< [IN] Whether the converted string should include braces */
);

#if defined(__cplusplus)
}
#endif

/** @}*/
